#include "SD.h"
#include <devices/audio.h>
#include <devices/narrator.h>
#include <proto/translator.h>

#define PHONEMESSIZE 512

static BYTE AllocTries[]={ 1,2,4,8 }; /* we try all channels */

static unsigned char SoundOpen, Volume = 32;
static struct MsgPort *SoundPort;
static struct IOAudio *IOAudio;
static BYTE chip sound[]={ 0,64,127,64,0,-64,-127,-64 }; /* a very little sound */

static unsigned char VoiceOpen;
static struct MsgPort *VoicePort;
static unsigned char *Phonemes;
static struct narrator_rb *rb;
static struct Library *TranslatorBase;

/*
 * The following functions manage the sound (beeping) system. They are called
 * *exclusively* from the copy task. The only function which is allowed from the
 * main task is SetVolume().
 */


/*
 * This function sets the volume.
 */

unsigned char SetVolume(int n) {
	if (n<0 || n>64) return(TRUE);
	Volume = (unsigned char)n;
	return(FALSE);
}


/*
 * This function produces an async beep. If Flash is TRUE, it also DisplayBeep()s.
 */

void Beep(unsigned short Period, unsigned char Flash) {
	if (SoundOpen && Volume) {
		WaitPort(SoundPort);
		GetMsg(SoundPort);
		IOAudio->ioa_Volume = Volume;
		IOAudio->ioa_Period = Period;
		BeginIO(IOAudio);
	}
	if (Flash) DisplayBeep(NULL);
}

/*
 * Here we open the audio.device. Nothing is returned because we don't really
 * care if we can use audio.device.
 */

void SetUpAudio(void) {

	if (SoundOpen) return;


	if ((SoundPort = CreatePort(NULL,0)) &&
		(IOAudio = (void *)CreateExtIO(SoundPort, sizeof(struct IOAudio)))) {

		IOAudio->ioa_Request.io_Message.mn_Node.ln_Pri = -50;
		IOAudio->ioa_Data = AllocTries;
		IOAudio->ioa_Length = sizeof(AllocTries);

		if (SoundOpen = !OpenDevice(AUDIONAME,0,IOAudio,0)) {
			IOAudio->ioa_Request.io_Command = CMD_WRITE;
			IOAudio->ioa_Request.io_Flags = ADIOF_PERVOL;
			IOAudio->ioa_Data = sound;
			IOAudio->ioa_Length = sizeof(sound);
			IOAudio->ioa_Period = 400;
			IOAudio->ioa_Cycles = 8;
			BeginIO(IOAudio);
			return;
		}
	}

	CloseAudio();
}

/*
 * Here we close the audio.device
 */

void CloseAudio(void) {

	if (SoundOpen) {
		WaitPort(SoundPort);
		GetMsg(SoundPort);
		CloseDevice(IOAudio);
		SoundOpen = 0;
	}
	if (IOAudio) {
		DeleteExtIO(IOAudio);
		IOAudio = NULL;
	}
	if (SoundPort) {
		DeletePort(SoundPort);
		SoundPort = NULL;
	}
}

/*
 * The following functions manage the voice system. They are called
 * *exclusively* from the main task.
 */

/*
 * Here we open the narrator.device, if someone selected Talk.
 */

int OpenVoice(void) {

	if (VoiceOpen) return(TRUE);

	if ((VoicePort = CreatePort(NULL,0)) &&
		(rb = (void *)CreateExtIO(VoicePort, sizeof(struct narrator_rb))) &&
		(TranslatorBase = OpenLibrary("translator.library",0)) &&
		(Phonemes = AllocMem(PHONEMESSIZE, MEMF_PUBLIC | MEMF_CLEAR)) &&
		(VoiceOpen = !OpenDevice("narrator.device", 0, rb, 0))) {
		rb->message.io_Command = CMD_WRITE;
		rb->message.io_Data = (APTR)Phonemes;
		rb->ch_masks = AllocTries;
		rb->nm_masks = 4;
		return(TRUE);
	}

	CloseVoice();
	return(FALSE);
}

/*
 * Here we close the narrator.device.
 */

void CloseVoice(void) {

	if (VoiceOpen) {
		if (VoiceOpen == 2) WaitIO(rb);
		CloseDevice(rb);
		VoiceOpen = 0;
	}

	if (rb) {
		DeleteExtIO(rb);
		rb = NULL;
	}

	if (VoicePort) {
		DeletePort(VoicePort);
		VoicePort = NULL;
	}

	if (TranslatorBase) {
		CloseLibrary(TranslatorBase);
		TranslatorBase = NULL;
	}

	if (Phonemes) {
		FreeMem(Phonemes, PHONEMESSIZE);
		Phonemes = NULL;
	}
}

/*
 * And here we talk!
 * Note that Say() is *always* called---only it won't to anything if VoiceOpen==0.
 * Note also that since we use SendIO() the spoken message and the system
 * requester run concurrently 8^).
 */

void Say(unsigned char *Msg) {

	if (VoiceOpen) {
		if (VoiceOpen == 1) VoiceOpen = 2;
		else WaitIO(rb);
		Translate(Msg, strlen(Msg), Phonemes, PHONEMESSIZE);
		rb->message.io_Length = strlen(Phonemes);
		SendIO(rb);
	}
}
